from django.test import TestCase

from django_spire.ai.sms.models import SmsConversation


class SmsModelTests(TestCase):
    def test_conversation_creation(self):
        conversation = SmsConversation.objects.create(
            phone_number='+15551234567'
        )
        self.assertEqual(conversation.phone_number, '+15551234567')
        self.assertEqual(conversation.messages.count(), 0)
        self.assertTrue(conversation.is_empty)

    def test_add_message(self):
        conversation = SmsConversation.objects.create(
            phone_number='+15551234567'
        )
        
        message = conversation.add_message(body="Hello", is_inbound=True, twilio_sid='')
        self.assertEqual(message.body, "Hello")
        self.assertTrue(message.is_inbound)

        message = conversation.add_message(body="Hi there", is_inbound=False, twilio_sid='')
        self.assertEqual(message.body, "Hi there")
        self.assertFalse(message.is_inbound)

        self.assertEqual(conversation.messages.count(), 2)
        self.assertFalse(conversation.is_empty)




from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.shortcuts import get_object_or_404

from django_spire.auth.group import models
from django_spire.auth.permissions.decorators import permission_required
from django_spire.auth.permissions.tools import generate_group_perm_data
from django_spire.contrib.generic_views import portal_views
from django_spire.contrib.pagination.pagination import paginate_list

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


@permission_required('django_spire_auth_group.view_authgroup')
def detail_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_404(models.AuthGroup, pk=pk)

    active_user_list = group.user_set.filter(is_active=True).order_by('first_name', 'last_name')
    inactive_user_list = group.user_set.filter(is_active=False).order_by('first_name', 'last_name')

    paginated_active_user_list = paginate_list(active_user_list, page_number=request.GET.get('page', 1), per_page=10)
    paginated_inactive_user_list = paginate_list(inactive_user_list, page_number=request.GET.get('page', 1), per_page=10)

    context_data = {
        'group': group,
        'permission_data': generate_group_perm_data(group, with_special_role=True),
        'active_user_list': paginated_active_user_list,
        'inactive_user_list': paginated_inactive_user_list,
    }

    return portal_views.detail_view(
        request,
        context_data=context_data,
        obj=group,
        template='django_spire/auth/group/page/detail_page.html'
    )


@permission_required('django_spire_auth_group.view_authgroup')
def list_view(request: WSGIRequest) -> TemplateResponse:
    group_list = models.AuthGroup.objects.all().prefetch_related('permissions').order_by('name')

    context_data = {
        'group_list': group_list,
        'group_list_permission_data': [
            generate_group_perm_data(group)
            for group in group_list
        ]
    }

    return portal_views.list_view(
        request,
        context_data=context_data,
        model=models.AuthGroup,
        template='django_spire/auth/group/page/list_page.html'
    )



from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.timezone import localtime

from django_spire.history.activity.querysets import ActivityQuerySet


class Activity(models.Model):
    content_type = models.ForeignKey(
        ContentType,
        related_name='activities',
        related_query_name='activity',
        on_delete=models.CASCADE, 
        editable=False
    )
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        User,
        related_name='users',
        related_query_name='user',
        on_delete=models.CASCADE,
        editable=False
    )

    recipient = models.ForeignKey(
        User,
        related_name='actors',
        related_query_name='actor',
        on_delete=models.CASCADE,
        editable=False,
        blank=True,
        null=True
    )

    verb = models.CharField(max_length=64)
    information = models.TextField(null=True, blank=True)

    created_datetime = models.DateTimeField(default=localtime)

    objects = ActivityQuerySet.as_manager()

    def __str__(self):
        return f'{self.user} - {self.verb}'

    def add_subscriber(self, subscriber: User) -> None:
        self.subscribers.create(
            user=subscriber
        )

    class Meta:
        db_table = 'django_spire_history_activity'
        verbose_name = 'Activities'
        verbose_name_plural = 'Activities'
        ordering = ['-created_datetime']


class ActivitySubscriber(models.Model):
    activity = models.ForeignKey(
        'django_spire_history_activity.Activity',
        on_delete=models.CASCADE,
        related_name='subscribers',
        related_query_name='subscriber'
    )

    subscriber = models.ForeignKey(
        User,
        related_name='activity_subscribers',
        related_query_name='activity_subscriber',
        on_delete=models.CASCADE,
        editable=False
    )

    created_datetime = models.DateTimeField(default=localtime)

    def __str__(self):
        return f'{self.activity} - {self.subscriber}'
    
    class Meta:
        db_table = 'django_spire_history_activity_subscriber'
        verbose_name = 'Activity Subscriber'
        verbose_name_plural = 'Activity Subscribers'
        ordering = ['-created_datetime']



from __future__ import annotations

import os
import threading
import time

from pathlib import Path
from typing_extensions import Any, Callable, TYPE_CHECKING

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

try:
    from pyinstrument import Profiler
except ImportError:
    Profiler = None

if TYPE_CHECKING:
    from django.http import HttpRequest, HttpResponse


IGNORE_PATH = [
    '/__',
    '/__debug__/',
    '/__debug__/history_sidebar/',
    '/__reload__/events/',
    '/_admin_profiling/',
    '/admin/',
    '/admin/autocomplete/',
    '/admin/jsi18n/',
    '/api-auth/',
    '/api-auth/login/',
    '/api-auth/logout/',
    '/api/schema/',
    '/browsable-api/',
    '/debug/',
    '/debug-toolbar/',
    '/django_glue/',
    '/docs/',
    '/favicon.ico',
    '/media/',
    '/openapi/',
    '/redoc/',
    '/robots.txt',
    '/schema/',
    '/sitemap.xml',
    '/static/',
    '/swagger/',
    'django_glue',
]


IGNORE_EXTENSION = [
    '.css',
    '.eot',
    '.gif',
    '.ico',
    '.jpeg',
    '.jpg',
    '.js',
    '.map',
    '.pdf',
    '.png',
    '.svg',
    '.ttf',
    '.txt',
    '.woff',
    '.woff2',
    '.zip',
]


class ProfilingMiddleware(MiddlewareMixin):
    def __init__(self, get_response: Callable[[HttpRequest], HttpResponse]) -> None:
        super().__init__(get_response)

        if Profiler is None:
            message = 'pyinstrument is required for profiling.'
            raise ImportError(message)

        configuration = {
            'PROFILING_DIR': os.getenv('PROFILING_DIR', '.profile'),
            'PROFILING_ENABLED': os.getenv('PROFILING_ENABLED', 'False') == 'True',
        }

        directory = configuration.get('PROFILING_DIR', '.profile')

        if isinstance(directory, str):
            if not Path(directory).is_absolute():
                current = Path.cwd()
                base = getattr(settings, 'BASE_DIR', current)
                directory = Path(base) / directory
            else:
                directory = Path(directory)

        self.directory = Path(directory)
        self.directory.mkdir(exist_ok=True)

        self.enabled = configuration.get('PROFILING_ENABLED', False)
        self.profile_threshold = configuration.get('PROFILE_THRESHOLD', 0)

        self.count = 0
        self.lock = threading.Lock()

    def _remove_profile(self) -> None:
        files = self.directory.glob('*.html')
        profiles = list(files)

        profiles.sort(
            key=lambda p: p.stat().st_mtime,
            reverse=True
        )

        maximum = 10

        if len(profiles) > maximum:
            for profile in profiles[maximum:]:
                profile.unlink()

    def _save_profile(
        self,
        profiler: Profiler,
        request: HttpRequest,
        duration_ms: float
    ) -> None:
        with self.lock:
            timestamp = int(time.time() * 1000)
            method = request.method
            path = request.path.replace('/', '_').replace('.', '_')

            if not path or path == '_':
                path = 'root'

            filename = f'{timestamp}_{method}_{path}_{duration_ms:.1f}ms_{request._profiling_id}.html'

            path = str(self.directory / filename)
            profiler.write_html(path)

            self._remove_profile()

    def _should_skip_profiling(self, request: HttpRequest) -> bool:
        path = request.path

        return (
            any(path.startswith(pattern) or pattern in path for pattern in IGNORE_PATH) or
            any(path.endswith(extension) or extension in path for extension in IGNORE_EXTENSION)
        )

    def process_view(
        self,
        request: HttpRequest,
        view_func: Callable[..., Any],
        args: tuple[Any, ...],
        kwargs: dict[str, Any]
    ) -> Any:
        if not settings.DEBUG:
            return None

        if not self.enabled:
            return None

        if self._should_skip_profiling(request):
            return None

        with self.lock:
            self.count = self.count + 1
            request._profiling_id = self.count

        profiler = Profiler(interval=0.001)
        start_time = time.time()
        profiler.start()

        try:
            response = view_func(request, *args, **kwargs)

            if hasattr(response, 'render'):
                response.render()
        except Exception:
            profiler.stop()
            duration_ms = (time.time() - start_time) * 1000
            self._save_profile(profiler, request, duration_ms)

            raise
        else:
            profiler.stop()
            duration_ms = (time.time() - start_time) * 1000

            if duration_ms >= self.profile_threshold:
                self._save_profile(profiler, request, duration_ms)

            return response



# from django.urls.conf import path, include
#
#
# app_name = 'spirechildapp'
#
# urlpatterns = [
#     path('page/', include('module.urls.page_urls', namespace='page')),
#     path('form/', include('module.urls.form_urls', namespace='form')),
# ]



from dandy.llm import Prompt
from django.db.models.enums import TextChoices
from django.db.models.fields import Field


def model_field_choices_prompt(
        model_field: Field,
        choices: TextChoices,
) -> Prompt:
    model_name = model_field.field.model.__name__
    field_name = model_field.field.name
    return (
        Prompt()
        .heading(f'Generate values for {field_name} in {model_name} model')
        .text(f'The value for the {field_name} field should only be selected from the following list of choices.')
        .list([choice[0] for choice in choices])
    )



from django.urls import include, path

from django_spire.knowledge.collection import views

app_name = 'collection'

urlpatterns = [
    path('page/', include('django_spire.knowledge.collection.urls.page_urls', namespace='page')),
    path('json/', include('django_spire.knowledge.collection.urls.json_urls', namespace='json')),
    path('form/', include('django_spire.knowledge.collection.urls.form_urls', namespace='form')),
]



from __future__ import annotations

from django.contrib.auth.decorators import login_required
from typing_extensions import TYPE_CHECKING

from django_spire.contrib.generic_views import portal_views

from django_spire.notification.app.models import AppNotification

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


@login_required()
def app_notification_list_view(request: WSGIRequest) -> TemplateResponse:
    app_notification_list = (
        AppNotification.objects.active()
        .is_sent()
        .annotate_is_viewed_by_user(request.user)
        .select_related('notification')
        .distinct()
        .ordered_by_priority_and_sent_datetime()
    )

    return portal_views.list_view(
        request,
        context_data={'notification_list': app_notification_list},
        model=AppNotification,
        template='django_spire/notification/app/page/list_page.html'
    )



from django_spire.ai.prompt.tuning.bots import SimplePromptTuningBot
from dandy.recorder import Recorder

from django_spire.ai.prompt.bots import DandyPythonPromptBot
from django_spire.ai.prompt.system import bots

def create_system_prompt_cli():
    Recorder.start_recording(recording_name='system_prompt')

    user_story = input('Enter your user story: ')
    prompt = bots.SystemPromptBot.process(user_story)
    print(prompt)

    while True:
        print("\nEnter your feedback (or type 'stop' to finish):")
        feedback = input()

        if feedback == "stop":
            break

        print('Creating new prompt from feedback.....')
        new_prompt = (
            SimplePromptTuningBot()
            .process(prompt, feedback)
        )
        print('----------------------------------------------------')
        print()
        print(new_prompt.prompt)
        prompt = new_prompt.prompt
        print()

    Recorder.stop_recording(recording_name='system_prompt')
    Recorder.to_html_file(recording_name='system_prompt')

    prompt_file = input('Do you want to create a Prompt File y/n?')
    if prompt_file.strip().lower() == 'y':
        DandyPythonPromptBot().process(prompt)



from django_spire.contrib.generic_views.modal_views import dispatch_modal_delete_form_content
from django_spire.contrib.generic_views.portal_views import template_view

__all__ = [
    'dispatch_modal_delete_form_content',
    'template_view',
]


from dandy.llm import BaseLlmBot

from module.intelligence import prompts
from module.intelligence import intel


class SpireChildAppBot(BaseLlmBot):
    instructions_prompt = prompts.spirechildapp_instruction_prompt()
    intel_class = intel.SpireChildAppIntel

    @classmethod
    def process(
            cls,
            user_input: str
    ) -> intel.SpireChildAppIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.spirechildapp_user_input_prompt(user_input)
        )



from django_spire.auth.controller.controller import BaseAuthController


class BaseAiChatAuthController(BaseAuthController):
    def can_delete(self):
        return self.request.user.has_perm('django_spire_ai_chat.delete_chat')



from __future__ import annotations

from django.contrib.auth import views as auth_views
from django.urls import reverse_lazy

from django_spire.contrib.form.utils import show_form_errors


class LoginView(auth_views.LoginView):
    template_name = 'django_spire/auth/page/login_page.html'

    def form_invalid(self, form):
        show_form_errors(self.request, form)
        return super().form_invalid(form)


class PasswordChangeView(auth_views.PasswordChangeView):
    template_name = 'django_spire/auth/page/password_change_page.html'
    success_url = reverse_lazy('django_spire:auth:admin:password_change_done')


class PasswordChangeDone(auth_views.PasswordChangeDoneView):
    template_name = 'django_spire/auth/page/password_change_done_page.html'


class PasswordResetView(auth_views.PasswordResetView):
    email_template_name = 'django_spire/auth/password_reset_email.html'
    success_url = reverse_lazy('django_spire:auth:admin:password_reset_done')
    template_name = 'django_spire/auth/page/password_reset_page.html'


class PasswordResetComplete(auth_views.PasswordResetCompleteView):
    template_name = 'django_spire/auth/page/password_reset_complete_page.html'


class PasswordResetConfirmation(auth_views.PasswordResetConfirmView):
    success_url = reverse_lazy('django_spire:auth:admin:password_reset_complete')
    template_name = 'django_spire/auth/page/password_reset_confirmation_page.html'


class PasswordResetDone(auth_views.PasswordResetDoneView):
    template_name = 'django_spire/auth/page/password_reset_done_page.html'


class PasswordResetKeyForm(auth_views.PasswordResetView):
    template_name = 'django_spire/auth/page/password_reset_key_form_page.html'


class PasswordResetKeyFormDone(auth_views.PasswordResetView):
    template_name = 'django_spire/auth/page/password_reset_key_done_page.html'


class PasswordSetForm(auth_views.PasswordResetView):
    template_name = 'django_spire/auth/page/password_set_form_page.html'



from __future__ import annotations

from abc import abstractmethod
from typing import Type

from django.core.handlers.wsgi import WSGIRequest
from django.db.models import QuerySet
from django.forms import Form

from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.queryset.enums import SessionFilterActionEnum
from django_spire.contrib.session.controller import SessionController


class SessionFilterQuerySetMixin(QuerySet):

    def process_session_filter(
            self,
            request: WSGIRequest,
            session_key: str,
            form_class: Type[Form]
    ) -> QuerySet:
        # Session keys must match to process new queryset data

        action = request.GET.get('action')
        form = form_class(request.GET)

        if form.is_valid():
            session = SessionController(request=request, session_key=session_key)

            # Todo: Change actions into an enum
            if action == SessionFilterActionEnum.CLEAR.value:
                session.purge()
                return self

            # The user has submitted the filter form
            if action == SessionFilterActionEnum.FILTER.value and session_key == request.GET.get('session_filter_key'):

                # Update session data
                for key, value in form.cleaned_data.items():
                    session.add_data(key, value)

            return self.bulk_filter(session.data)
        else:
            show_form_errors(request, form)
            return self


    @abstractmethod
    def bulk_filter(self, filter_data: dict) -> QuerySet:
        pass


class SearchQuerySetMixin(QuerySet):

    @abstractmethod
    def search(self, value: str | None) -> QuerySet:
        pass



import re

from django import template

register = template.Library()


@register.filter
def format_to_html(value: str) -> str:
    if not value:
        return '&nbsp;'

    line_break_html = re.sub(r'\n', '<br>', value)
    bolded_html = re.sub(
        r'\*\*(.*?)\*\*', r'<span class="fw-bold">\1</span>',
        line_break_html
    )
    italicized_html = re.sub(
        r'\*(.*?)\*', r'<span class="fst-italic">\1</span>',
        bolded_html
    )
    return re.sub(
        r'~~(.*?)~~', r'<span class="text-decoration-line-through">\1</span>',
        italicized_html
    )



from __future__ import annotations

from dandy.intel import BaseIntel

from django_spire.ai.prompt.bots import DandyPythonPromptBot, TextToMarkdownPromptBot
from django_spire.knowledge.entry.version.intelligence.bots.markdown_format_llm_bot import MarkdownFormatLlmBot


class SystemPromptResultIntel(BaseIntel):
    result: str


class SystemPromptIntel(BaseIntel):
    role: str
    task: str
    guidelines: str
    output_format: str | None

    def to_markdown(self):
        return TextToMarkdownPromptBot.process(self.to_string())

    def to_python(self):
        return DandyPythonPromptBot.process(self.to_string())

    def to_string(self):
        prompt = (
            f"Role: {self.role}\n"
            f"Task: {self.task}\n"
            f"Guidelines: {self.guidelines}\n"
        )

        if self.output_format:
            prompt += f"Output Format: {self.output_format}"

        return prompt



from typing import List

from dandy.intel import BaseIntel, BaseListIntel

from django_spire.knowledge.intelligence.intel.collection_intel import CollectionIntel


class EntryIntel(BaseIntel):
    body: str
    collection_intel: CollectionIntel


class EntriesIntel(BaseListIntel[EntryIntel]):
    entry_intel_list: List[EntryIntel]


from dandy.llm import BaseLlmBot, LlmConfigOptions

from django_spire.ai.prompt.system import prompts
from django_spire.ai.prompt.system import intel
from django_spire.ai.prompt.system.intel import SystemPromptIntel


class RoleSystemPromptBot(BaseLlmBot):
    instructions_prompt = prompts.role_bot_prompt()
    intel_class = intel.SystemPromptResultIntel
    config_options = LlmConfigOptions(
        temperature=0.5
    )

    @classmethod
    def process(
            cls,
            user_story: str

    ) -> intel.SystemPromptResultIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story)
        )


class TaskSystemPromptBot(BaseLlmBot):
    instructions_prompt = prompts.task_bot_prompt()
    intel_class = intel.SystemPromptResultIntel
    config_options = LlmConfigOptions(
        temperature=0.5
    )

    @classmethod
    def process(
            cls,
            user_story: str

    ) -> intel.SystemPromptResultIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story)
        )


class GuidelinesSystemPromptBot(BaseLlmBot):
    instructions_prompt = prompts.guidelines_bot_prompt()
    intel_class = intel.SystemPromptResultIntel
    config_options = LlmConfigOptions(
        temperature=0.2
    )

    @classmethod
    def process(
            cls,
            user_story: str

    ) -> intel.SystemPromptResultIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story)
        )


class OutputFormatSystemPromptBot(BaseLlmBot):
    instructions_prompt = prompts.output_format_bot_prompt()
    intel_class = intel.SystemPromptResultIntel
    config_options = LlmConfigOptions(
        temperature=0.5
    )

    @classmethod
    def process(
            cls,
            user_story: str

    ) -> intel.SystemPromptResultIntel:

        return cls.process_prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story)
        )


class SystemPromptBot(BaseLlmBot):
    instructions_prompt = prompts.system_prompt_instruction_bot_prompt()
    intel_class = intel.SystemPromptIntel


    @classmethod
    def process(
            cls,
            user_story: str

    ) -> intel.SystemPromptIntel:

        role_future = RoleSystemPromptBot.process_to_future(user_story)
        task_future = TaskSystemPromptBot.process_to_future(user_story)
        guidelines_future = GuidelinesSystemPromptBot.process_to_future(user_story)
        output_format_future = OutputFormatSystemPromptBot.process_to_future(user_story)

        role = role_future.result
        task = task_future.result
        guidelines = guidelines_future.result
        output_format = output_format_future.result

        return SystemPromptIntel(
            role=role.result,
            task=task.result,
            guidelines=guidelines.result,
            output_format=output_format.result
        )



from __future__ import annotations

from django.utils.timezone import localdate

from dandy.llm import LlmBot, LlmConfigOptions, Prompt


class LlmFieldSeedingBot(LlmBot):
    config = 'SEEDING_LLM_BOT'

    config_options = LlmConfigOptions(
        randomize_seed=True,
        temperature=0.5
    )

    instructions_prompt = (
        Prompt()
        .title('You are a database seeding bot.')
        .text('Below you will find rules and instructions.')
        .text('Rules are specific per field and must be followed.')
        .text('ALL FIELDS ARE REQUIRED TO HAVE DATA.')
        .text('Instructions have context behind the meaning of the data and how it should be created.')
        .text(f'Today\'s date is {localdate().strftime("%Y-%m-%d")} use this in context for generating dates and datetimes')
    )

    @classmethod
    def process(cls, prompt: Prompt, intel_class) -> list[dict]:
        intel_data = cls.process_prompt_to_intel(
            prompt=prompt,
            intel_class=intel_class
        )
        return intel_data.model_dump(mode='json')['items']



# Generated by Django 5.2.5 on 2025-09-06 00:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_knowledge', '0002_alter_entryversionblock_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collection',
            name='order',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='entry',
            name='order',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='entryversionblock',
            name='order',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



from django.contrib.sites.models import Site
from django.db import models
from django.urls import reverse
from django.utils.timezone import now

from django_spire.notification.models import Notification
from django_spire.notification.sms.choices import SmsMediaContentTypeChoices
from django_spire.notification.sms.querysets import SmsNotificationQuerySet, \
    SmsTemporaryMediaQuerySet


class SmsTemporaryMedia(models.Model):
    content = models.TextField()
    content_type = models.CharField(max_length=64, choices=SmsMediaContentTypeChoices)
    name = models.CharField(max_length=255)
    expire_datetime = models.DateTimeField()
    external_access_key = models.UUIDField(unique=True)

    objects = SmsTemporaryMediaQuerySet.as_manager()

    def __str__(self):
        return f'{self.name} - {self.content_type}'

    @property
    def external_url(self) -> str:
        path = reverse(
            'django_spire:notification:sms:media:temporary_media',
            kwargs={'external_access_key': self.external_access_key},
        )[1:]

        return f'{Site.objects.get_current()}/{path}'

    def is_expired(self) -> bool:
        return self.expire_datetime < now()

    def is_ready_for_deletion(self) -> bool:
        return not self.has_unsent_notifications() or self.is_expired()

    def has_unsent_notifications(self) -> bool:
        return self.sms_notifications.notification.unsent().active().exists()

    class Meta:
        db_table = 'django_spire_notification_sms_temporary_media'
        verbose_name = 'SMS Temporary Media'
        verbose_name_plural = 'SMS Temporary Media'


class SmsNotification(models.Model):
    notification = models.OneToOneField(
        Notification,
        editable=False,
        on_delete=models.CASCADE,
        related_name='sms',
        related_query_name='sms',
    )
    temporary_media = models.ForeignKey(
        SmsTemporaryMedia,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name='sms_notifications',
        related_query_name='sms_notification',
    )
    to_phone_number = models.CharField(max_length=11, blank=True)
    media_url = models.URLField(max_length=255, blank=True, null=True)

    objects = SmsNotificationQuerySet.as_manager()

    def __str__(self):
        return f'{self.to_phone_number} - {self.notification.title}'

    class Meta:
        db_table = 'django_spire_notification_sms'
        verbose_name = 'SMS Notification'
        verbose_name_plural = 'SMS Notifications'



from __future__ import annotations

import json

from django.test import RequestFactory

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.core.shortcuts import (
    get_object_or_none,
    get_object_or_null_obj,
    model_object_from_app_label,
    process_request_body
)

# from testing.dummy.models import DummyModel
#
#
# class ShortcutsTestCase(BaseTestCase):
#     def setUp(self) -> None:
#         super().setUp()
#
#         self.dummy = DummyModel.objects.create(name='test')
#         self.request_factory = RequestFactory()
#
#     def test_get_object_or_null_obj_queryset_found(self) -> None:
#         obj = get_object_or_null_obj(DummyModel.objects.all(), name='test')
#         self.assertEqual(obj, self.dummy)
#
#     def test_get_object_or_null_obj_queryset_not_found(self) -> None:
#         obj = get_object_or_null_obj(DummyModel.objects.all(), name='nonexistent')
#         self.assertIsNone(obj.pk)
#         self.assertEqual(obj.name, '')
#
#     def test_get_object_or_null_obj_model_found(self) -> None:
#         obj = get_object_or_null_obj(DummyModel, name='test')
#         self.assertEqual(obj, self.dummy)
#
#     def test_get_object_or_null_obj_model_not_found(self) -> None:
#         obj = get_object_or_null_obj(DummyModel, name='nonexistent')
#         self.assertIsNone(obj.pk)
#         self.assertEqual(obj.name, '')
#
#     def test_get_object_or_none_found(self) -> None:
#         obj = get_object_or_none(DummyModel, pk=self.dummy.pk)
#         self.assertEqual(obj, self.dummy)
#
#     def test_get_object_or_none_not_found(self) -> None:
#         obj = get_object_or_none(DummyModel, pk=9999)
#         self.assertIsNone(obj)
#
#     def test_process_request_body(self) -> None:
#         data = {'data': {'key': 'value'}}
#
#         request = self.request_factory.post(
#             '/dummy-url/',
#             data=json.dumps(data),
#             content_type='application/json'
#         )
#
#         result = process_request_body(request)
#         self.assertEqual(result, data['data'])
#
#     def test_model_object_from_app_label_found(self) -> None:
#         obj = model_object_from_app_label('dummy', 'dummymodel', self.dummy.pk)
#         self.assertEqual(obj, self.dummy)
#
#     def test_model_object_from_app_label_not_found(self) -> None:
#         obj = model_object_from_app_label('dummy', 'dummymodel', 9999)
#         self.assertIsNone(obj)
#
#     def test_model_object_from_app_label_contenttype_not_found(self) -> None:
#         obj = model_object_from_app_label('nonexistent', 'nonexistent', 1)
#         self.assertIsNone(obj)



import time
from unittest.mock import MagicMock, patch

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.constants import MARKDOWN_AI_CHUNK_SIZE
from django_spire.knowledge.entry.version.converters.docx_converter import DocxConverter

from django_spire.knowledge.entry.version.tests.factories import \
    create_test_entry_version


class DocxConverterTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.test_entry_version = create_test_entry_version()

    @patch(
        'django_spire.knowledge.entry.version.converters.docx_converter.'
        'MarkdownFormatLlmBot.process'
    )
    def test_improve_markdown_structure_concurrent_execution(
            self,
            mock_process_method: MagicMock
    ):
        def slow_process(markdown_content: str):
            time.sleep(0.1)
            return markdown_content

        mock_process_method.side_effect = slow_process
        docx_converter = DocxConverter(entry_version=self.test_entry_version)
        mock_chunks = [
            f'{x}' + ''.join('x' for _ in range(MARKDOWN_AI_CHUNK_SIZE - 1))
            for x in range(1, 4)
        ]
        start_time = time.time()
        improved_markdown = docx_converter.improve_markdown_structure(
            markdown_content=''.join(mock_chunks)
        )
        end_time = time.time()
        self.assertLess(end_time - start_time, 0.3)
        self.assertEqual(mock_process_method.call_count, 3)
        self.assertEqual(improved_markdown, ''.join(mock_chunks))



from collections import defaultdict

from django.contrib.admin.options import get_content_type_for_model
from django.contrib.auth.models import User
from django.db.models import QuerySet, Q, Model
from django.utils.timezone import now

from django_spire.history.querysets import HistoryQuerySet
from django_spire.notification.choices import (
    NotificationStatusChoices,
    NotificationTypeChoices,
    NotificationPriorityChoices,
)


class NotificationQuerySet(HistoryQuerySet):
    def app_notifications(self):
        return self.filter(type=NotificationTypeChoices.APP)

    def by_content_object(self, content_object: type[Model]):
        return self.filter(
            content_type=get_content_type_for_model(content_object),
            object_id=content_object.pk,
        )

    def by_content_objects(self, content_objects: list[type[Model]]):
        if not content_objects:
            return self.none()

        content_type_to_object_ids = defaultdict(list)

        for content_object in content_objects:
            content_type_to_object_ids[
                get_content_type_for_model(content_object)
            ].append(content_object.pk)

        queries = Q()
        for content_type, object_ids in content_type_to_object_ids.items():
            queries |= Q(
                content_type=content_type,
                object_id__in=object_ids
            )

        return self.filter(queries)

    def by_user(self, user: User):
        return self.filter(user=user)

    def by_users(self, users: list[User]):
        return self.filter(user__in=users)

    def email_notifications(self):
        return self.filter(type=NotificationTypeChoices.EMAIL)

    def errored(self):
        return self.filter(status=NotificationStatusChoices.ERRORED)

    def failed(self):
        return self.filter(status=NotificationStatusChoices.FAILED)

    def high_priority(self):
        return self.filter(priority=NotificationPriorityChoices.HIGH)

    def low_priority(self):
        return self.filter(priority=NotificationPriorityChoices.LOW)

    def medium_priority(self):
        return self.filter(priority=NotificationPriorityChoices.MEDIUM)

    def pending(self):
        return self.filter(status=NotificationStatusChoices.PENDING)

    def processing(self):
        return self.filter(status=NotificationStatusChoices.PROCESSING)

    def push_notifications(self):
        return self.filter(status=NotificationTypeChoices.PUSH)

    def ready_to_send(self):
        return self.pending().filter(publish_datetime__lte=now())

    def sent(self):
        return self.filter(status=NotificationStatusChoices.SENT)

    def sms_notifications(self):
        return self.filter(type=NotificationTypeChoices.SMS)

    def unsent(self):
        return self.filter(status__in=[NotificationStatusChoices.PENDING, NotificationStatusChoices.PROCESSING])


class NotificationContentObjectQuerySet(QuerySet):
    def by_notification_content_object(self, content_object: type[Model]):
        return self.filter(
            notification__content_type=get_content_type_for_model(content_object),
            notification__object_id=content_object.pk,
        )

    def by_notification_content_objects(self, content_objects: list[type[Model]]):
        if not content_objects:
            return self.none()

        content_type_to_object_ids = defaultdict(list)

        for content_object in content_objects:
            content_type_to_object_ids[
                get_content_type_for_model(content_object)
            ].append(content_object.pk)

        queries = Q()
        for content_type, object_ids in content_type_to_object_ids.items():
            queries |= Q(
                notification__content_type=content_type,
                notification__object_id__in=object_ids
            )

        return self.filter(queries)



# These imports are for the migrations to work as a single app "django_spire_knowledge"

from django_spire.knowledge.collection.admin import *
from django_spire.knowledge.entry.admin import *
from django_spire.knowledge.entry.version.admin import *
from django_spire.knowledge.entry.version.block.admin import *



from __future__ import annotations

from django import template

register = template.Library()


@register.simple_tag()
def comment_form(related_obj, return_url, user, comment=None, parent=None, user_list=None):
    from django_spire.comment.forms import CommentForm

    return CommentForm(
        related_obj,
        return_url,
        user,
        instance=comment,
        parent=parent,
        user_list=user_list
    )


@register.simple_tag()
def user_list_from_content_type(related_obj):
    from django_spire.comment.utils import find_user_list_from_content_type

    return find_user_list_from_content_type(
        related_obj._meta.app_label,
        related_obj._meta.model_name
    )



from __future__ import annotations

from django.core.paginator import Paginator, Page


def paginate_list(
    object_list: list,
    page_number: int = 1,
    per_page: int = 50
) -> Page:
    paginator = Paginator(object_list, per_page)
    return paginator.get_page(page_number)



from pydantic import constr, condecimal, create_model
from pydantic.fields import Field

from typing import Tuple, Type, Any, Optional

from django.db import models
from django.core import validators

from django_spire.core.converters.to_enums import django_choices_to_enums
from django_spire.core.maps import MODEL_FIELD_TYPE_TO_TYPE_MAP


def django_to_pydantic_model(
        model_class: Type[models.Model],
        base_class: Type | None = None,
        include_fields: str | list| tuple | None = None,
        exclude_fields: str | list | tuple | None = None
):
    if not issubclass(model_class, models.Model):
        raise ValueError("model_class must be a subclass of django.db.models.Model")

    if include_fields is None:
        include_fields = []

    if exclude_fields is None:
        exclude_fields = []

    pydantic_fields = {}

    for model_field in model_class._meta.fields:
        field_name = model_field.attname

        if len(include_fields) > 0 and field_name not in include_fields:
            continue

        if len(exclude_fields) > 0 and field_name in exclude_fields:
            continue

        converter = DjangoToPydanticFieldConverter(model_field)
        pydantic_fields[field_name] = converter.build_field()

    return create_model(
        f'{model_class.__name__}',
        __base__=base_class,
        **pydantic_fields
    )


class DjangoToPydanticFieldConverter:

    def __init__(self, model_field: models.Field):
        self.model_field = model_field
        self.kwargs = {}

        self._build_metadata()
        self.field_type = self._get_pydantic_type()
        self._wrap_nullable()

    @staticmethod
    def bool_to_json_schema(value: bool):
        if value:
            return "true"
        else:
            return "false"

    @property
    def field_handlers(self):
        return {
            'CharField': self._build_char_field,
            'IntegerField': self._build_integer_field,
            'SmallIntegerField': self._build_integer_field,
            'DecimalField': self._build_decimal_field,
            'DateField': self._build_date_field,
            'DateTimeField': self._build_date_time_field,
        }

    def _base_type(self):
        return MODEL_FIELD_TYPE_TO_TYPE_MAP.get(
            self.model_field.get_internal_type(),
            str
        )

    def _build_char_field(self) -> Type:
        if self.model_field.max_length:
            return constr(max_length=self.model_field.max_length)
        return str

    def _build_date_field(self) -> Type:
        self.kwargs['example'] = '2022-01-01'
        self.kwargs['json_schema_extra']['example'] = '2022-01-01'
        return self._base_type()

    def _build_date_time_field(self) -> Type:
        self.kwargs['example'] = '2022-01-01 13:37:00'
        self.kwargs['json_schema_extra']['example'] = '2022-01-01 13:37:00'
        return self._base_type()

    def _build_integer_field(self) -> Type:
        for validator in self.model_field.validators:
            if isinstance(validator, validators.MinValueValidator):
                self.kwargs['json_schema_extra']['greater_than'] = validator.limit_value
            elif isinstance(validator, validators.MaxValueValidator):
                self.kwargs['json_schema_extra']['less_than'] = validator.limit_value

        return self._base_type()

    def _build_decimal_field(self) -> Type:
        self.kwargs['json_schema_extra']['example'] = '0.00'
        self.kwargs['json_schema_extra']['max_digits'] = self.model_field.max_digits
        self.kwargs['json_schema_extra']['decimal_places'] = self.model_field.decimal_places

        return condecimal(
            max_digits=self.model_field.max_digits,
            decimal_places=self.model_field.decimal_places
        )

    def _build_enum_type(self):
        enum_name = f"{self.model_field.name.capitalize()}Enum"
        return django_choices_to_enums(enum_name, self.model_field.choices)

    def build_field(self) -> Tuple[Type, Any]:
        """Build and return the Pydantic field type and Field object."""
        return self.field_type, Field(**self.kwargs)

    def _build_metadata(self):

        # Cannot set a default when providing option to the LLM
        # if self.model_field.default is not models.NOT_PROVIDED:
        #     if callable(self.model_field.default):
        #         self.kwargs['default'] = self.model_field.default()
        #     else:
        #         self.kwargs['default'] = self.model_field.default

        self.kwargs['required'] = not self.model_field.null

        if self.model_field.null and self.model_field.default is models.NOT_PROVIDED:
            self.kwargs['default'] = None

        if self.model_field.help_text:
            self.kwargs['description'] = str(self.model_field.help_text)

        self.kwargs['json_schema_extra'] = {
            'is_unique': self.bool_to_json_schema(self.model_field.unique),
            # 'is_required': self.bool_to_json_schema(not self.model_field.null),
            'field_name': self.model_field.name
        }

        if self.model_field.choices:
            self.kwargs['json_schema_extra']['enum'] = [choice[0] for choice in self.model_field.choices]
            self.kwargs['description'] = f"Select a {self.model_field.name.lower()} category. Options: " + ", ".join(
                f"{value} ({label})" for value, label in self.model_field.choices
            ) + "."

    def _get_pydantic_type(self) -> Type:
        if self.model_field.choices:
            return self._build_enum_type()

        handler = self.field_handlers.get(self.model_field.get_internal_type())

        if handler:
            return handler()

        return self._base_type()

    def _wrap_nullable(self):
        pass
        # This works better for the LLM
        # if self.model_field.null:
        #     self.field_type = Optional[self.field_type]


# Generated by Django 5.2.1 on 2025-05-15 20:20

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('django_spire_notification', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppNotification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('template', models.TextField(default='django_spire/notification/app/item/notification_item.html')),
                ('context_data', models.JSONField(default=dict)),
                ('notification', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='app', related_query_name='app', to='django_spire_notification.notification')),
            ],
            options={
                'verbose_name': 'App Notification',
                'verbose_name_plural': 'App Notifications',
                'db_table': 'django_spire_notification_app',
            },
        ),
    ]



from django.contrib import admin
from django.shortcuts import reverse
from django.utils.html import format_html
from django.utils.http import urlencode

from django_spire.ai.chat import models

@admin.register(models.Chat)
class ChatAdmin(admin.ModelAdmin):
    list_display = ('name', 'user', 'view_chat_messages_link', 'created_datetime')
    search_fields = ('id', 'name')
    ordering = ['-id']

    def get_readonly_fields(self, request, obj=None):
        return [field.name for field in self.model._meta.fields]

    def view_chat_messages_link(self, obj):
        count = obj.messages.count()
        url = (
                reverse("admin:django_spire_ai_chat_chatmessage_changelist")
                + "?"
                + urlencode({"chat__id": f"{obj.id}"})
        )
        return format_html('<a href="%s">%s Messages</a>' % (url, count))

    view_chat_messages_link.short_description = "Messages"

    class Meta:
        ordering = ('id', )


@admin.register(models.ChatMessage)
class ChatMessageAdmin(admin.ModelAdmin):
    list_display = ('content_body', 'chat', 'chat__user','is_processed', 'is_viewed', 'created_datetime')
    search_fields = ('id', 'content')
    ordering = ['-id']

    def content_body(self, obj):
        return str(obj)

    content_body.short_description = 'Body'

    def get_readonly_fields(self, request, obj=None):
        return [field.name for field in self.model._meta.fields]




from __future__ import annotations

import tempfile

from django.test import Client, override_settings, TestCase

from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.tests.factories import create_super_user


TEMP_MEDIA_ROOT = tempfile.mkdtemp()


@override_settings(MEDIA_ROOT=TEMP_MEDIA_ROOT)
class BaseTestCase(TestCase):
    def setUp(self) -> None:
        self.client = Client()

        self.super_user = AuthUser.objects.create_superuser(
            username='stratus',
        )
        self.client.force_login(self.super_user)



import json

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.block.tests.factories import create_test_version_block


class EntryVersionBlockJsonUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_version_block = create_test_version_block()

    def test_update_text_view_url_path(self):
        response = self.client.post(
            reverse('django_spire:knowledge:entry:version:block:json:update_text'),
            data=json.dumps({
                'pk': self.test_version_block.pk,
                'value': '',
                'block_type': 'text'
            }),
            content_type='application/json'
        )

        self.assertEqual(response.status_code, 200)



from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskFormUrlsTestCase(BaseTestCase):
    def test_helpdesk_ticket_form_create_url_path(self) -> None:
        response = self.client.get(path=reverse('django_spire:help_desk:form:create'))

        self.assertEqual(response.status_code, 200)

    def test_helpdesk_ticket_form_update_url_path(self) -> None:
        response = self.client.get(
            path=reverse(
                'django_spire:help_desk:form:update',
                kwargs={'pk': create_test_helpdesk_ticket().pk}
            ),
        )

        self.assertEqual(response.status_code, 200)



from django_spire.ai.prompt.system import bots
from django_spire.core.tests.test_cases import BaseTestCase


class PromptBotTestCase(BaseTestCase):

    def test_system_prompt_system_prompt(self):
        ISSUE_PROMPT = (
            "I am using jira to plan developer workloads And I want to be able to build very clear issues  that"
            "developers will be able to read understand and take action on to complete the goal."
            "I will give you details about a software development task that we have to achieve and I want you "
            "to return a short one sentence summary that the developer can read and understand the scope of the issue"
            "before they get into the details of it "
        )

        role_bot = bots.RoleSystemPromptBot.process(ISSUE_PROMPT)
        self.assertIsNotNone(role_bot)

        task_bot = bots.TaskSystemPromptBot.process(ISSUE_PROMPT)
        self.assertIsNotNone(task_bot)


        guidelines_bot = bots.GuidelinesSystemPromptBot.process(ISSUE_PROMPT)
        self.assertIsNotNone(guidelines_bot)

        output_format_bot = bots.OutputFormatSystemPromptBot.process(ISSUE_PROMPT)
        self.assertIsNotNone(output_format_bot)
                


from django.utils.timezone import now
from sendgrid import SendGridException

from django_spire.notification.choices import NotificationTypeChoices, \
    NotificationStatusChoices
from django_spire.notification.email.helper import SendGridEmailHelper
from django_spire.notification.exceptions import NotificationException
from django_spire.notification.models import Notification
from django_spire.notification.processors.processor import BaseNotificationProcessor


class EmailNotificationProcessor(BaseNotificationProcessor):
    def process(self, notification: Notification):
        notification.status = NotificationStatusChoices.PROCESSING
        notification.save()

        try:
            if notification.type != NotificationTypeChoices.EMAIL:
                raise NotificationException(
                    f"EmailNotificationProcessor only processes "
                    f"Email notifications. Was provided {notification.type}"
                )

            SendGridEmailHelper(notification).send()

            notification.status = NotificationStatusChoices.SENT
            notification.sent_datetime = now()
        except Exception as e:
            notification.status_message = str(e)
            if isinstance(e, SendGridException):
                notification.status = NotificationStatusChoices.ERRORED
            else:
                notification.status = NotificationStatusChoices.FAILED
                raise e
        finally:
            notification.save()

    def process_list(self, notifications: list):
        self._update_notifications_to_processing(notifications)

        for notification in notifications:
            try:
                if notification.type != NotificationTypeChoices.EMAIL:
                    raise NotificationException(
                        f"EmailNotificationProcessor only processes "
                        f"Email notifications. Was provided {notification.type}"
                    )

                SendGridEmailHelper(notification).send()

                notification.status = NotificationStatusChoices.SENT
                notification.sent_datetime = now()
            except Exception as e:
                notification.status_message = str(e)
                if isinstance(e, SendGridException):
                    notification.status = NotificationStatusChoices.ERRORED
                else:
                    notification.status = NotificationStatusChoices.FAILED
                    Notification.objects.bulk_update(
                        notifications,
                        ['status', 'sent_datetime', 'status_message']
                    )
                    raise e

        Notification.objects.bulk_update(
            notifications,
            ['status', 'sent_datetime', 'status_message']
        )

    def process_ready(self):
        self.process_list(
            Notification.objects.
            email_notifications()
            .ready_to_send()
            .active()
            .prefetch_related('email')
        )

    def process_errored(self):
        self.process_list(
            Notification.objects
            .email_notifications()
            .errored()
            .active()
            .prefetch_related('email')
        )



from django.core.handlers.wsgi import WSGIRequest
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib import Breadcrumbs
from django_spire.knowledge.collection.models import Collection
from django_spire.knowledge.entry.models import Entry


@AppAuthController('knowledge').permission_required('can_view')
def file_list_view(request: WSGIRequest, collection_pk: int = 0) -> TemplateResponse:
    breadcrumbs = Breadcrumbs()
    breadcrumbs.add_breadcrumb(name='Knowledge')

    if collection_pk != 0:
        collection = Collection.objects.select_related('parent').get(pk=collection_pk)

        breadcrumbs.add_breadcrumb(
            name='Collections',
            href=reverse('django_spire:knowledge:collection:page:list')
        )

        if collection.parent_id is not None:
            parent = collection.parent
            breadcrumbs.add_breadcrumb(
                name=parent.name,
                href=reverse(
                    viewname='django_spire:knowledge:collection:page:detail',
                    kwargs={'pk': parent.pk}
                )
            )

        breadcrumbs.add_breadcrumb(
            name=collection.name,
            href=reverse(
                viewname='django_spire:knowledge:collection:page:detail',
                kwargs={'pk': collection.pk}
            )
        )
        breadcrumbs.add_breadcrumb(name='Importing Files')

    return TemplateResponse(
        request,
        context={
            'files_json': Entry.services.tool.get_files_to_convert_json(),
            'breadcrumbs': breadcrumbs,
        },
        template='django_spire/knowledge/entry/file/page/list_page.html'
    )



from __future__ import annotations
from typing import TYPE_CHECKING

from django_spire.contrib.service import BaseDjangoModelService

if TYPE_CHECKING:
    from module.models import SpireChildApp


class SpireChildAppIntelligenceService(BaseDjangoModelService['SpireChildApp']):
    obj: SpireChildApp



from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskPageUrlsTestCase(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_ticket = create_test_helpdesk_ticket()

    def test_ticket_delete_view_url_path(self) -> None:
        response = self.client.get(
            path=reverse(
                'django_spire:help_desk:page:delete',
                kwargs={'pk': self.test_ticket.pk}
            ),
        )

        self.assertEqual(response.status_code, 200)

    def test_ticket_detail_view_url_path(self) -> None:
        response = self.client.get(
            path=reverse(
                'django_spire:help_desk:page:detail',
                kwargs={'pk': self.test_ticket.pk}
            ),
        )

        self.assertEqual(response.status_code, 200)

    def test_ticket_list_view_url_path(self) -> None:
        response = self.client.get(path=reverse('django_spire:help_desk:page:list'))

        self.assertEqual(response.status_code, 200)



import json

from django.db import models
from django.utils.timezone import localtime

from django_spire.history.mixins import HistoryModelMixin
from django_spire.history.viewed.mixins import ViewedModelMixin
from django_spire.notification.app.querysets import AppNotificationQuerySet
from django_spire.notification.models import Notification


class AppNotification(ViewedModelMixin, HistoryModelMixin):
    notification = models.OneToOneField(
        Notification,
        editable=False,
        on_delete=models.CASCADE,
        related_name='app',
        related_query_name='app',
    )
    template = models.TextField(default='django_spire/notification/app/item/notification_item.html')
    context_data = models.JSONField(default=dict)

    objects = AppNotificationQuerySet.as_manager()

    def __str__(self):
        return f'{self.notification.title}'

    @property
    def verbose_time_since_delivered(self) -> str:
        delta = localtime() - self.notification.sent_datetime

        seconds = abs(delta.total_seconds())
        minutes = seconds // 60
        hours = minutes // 60
        days = hours // 24

        if days > 0:
            return f"{int(days)} day{'s' if days != 1 else ''} ago"
        elif hours > 0:
            return f"{int(hours)} hour{'s' if hours != 1 else ''} ago"
        elif minutes > 0:
            return f"{int(minutes)} minute{'s' if minutes != 1 else ''} ago"
        else:
            return "just now"

    def as_dict(self) -> dict:
        return {
            'id': self.id,
            'title': self.notification.title,
            'body': self.notification.body,
            'context_data': self.context_data,
            'priority': self.notification.priority,
            'url': self.notification.url,
            'time_since_delivered': self.verbose_time_since_delivered,
        }

    def as_json(self) -> str:
        return json.dumps(self.as_dict())

    class Meta:
        db_table = 'django_spire_notification_app'
        verbose_name = 'App Notification'
        verbose_name_plural = 'App Notifications'



from __future__ import annotations

from unittest.mock import patch
from typing_extensions import Any

from django.test import TestCase, RequestFactory

from django_spire.core.context_processors import theme_context
from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.theme.utils import get_theme_cookie_name


class ThemeContextProcessorTests(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()

    def test_theme_context_no_cookie(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}

        with patch('django_spire.conf.settings.DJANGO_SPIRE_DEFAULT_THEME', 'dracula-dark'):
            context = theme_context(request)

        self.assertEqual(context['DJANGO_SPIRE_DEFAULT_THEME'], 'dracula-dark')
        self.assertIn('theme', context)

        data = context['theme']
        self.assertEqual(data['family'], 'dracula')
        self.assertEqual(data['mode'], 'dark')

    def test_theme_context_with_cookie(self) -> None:
        request = self.factory.get('/')
        cookie_name = get_theme_cookie_name()
        request.COOKIES = {cookie_name: 'one-dark-light'}

        context = theme_context(request)

        self.assertIn('theme', context)
        data = context['theme']
        self.assertEqual(data['family'], 'one-dark')
        self.assertEqual(data['mode'], 'light')

    def test_theme_context_invalid_cookie(self) -> None:
        request = self.factory.get('/')
        cookie_name = get_theme_cookie_name()
        request.COOKIES = {cookie_name: 'invalid-theme'}

        with patch('django_spire.conf.settings.DJANGO_SPIRE_DEFAULT_THEME', 'dracula-dark'):
            context = theme_context(request)

        data = context['theme']
        self.assertEqual(data['family'], 'dracula')
        self.assertEqual(data['mode'], 'dark')

    def test_theme_context_path_setting(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}

        path = '/custom/path/{family}/{mode}.css'
        with patch('django_spire.conf.settings.DJANGO_SPIRE_THEME_PATH', path):
            context = theme_context(request)

        self.assertEqual(context['DJANGO_SPIRE_THEME_PATH'], path)


class ThemeContextProcessorIntegrationTests(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()
        self.factory = RequestFactory()

    def test_theme_context_with_authenticated_client(self) -> None:
        cookie_name = get_theme_cookie_name()
        self.client.cookies[cookie_name] = 'dracula-dark'
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

    def test_theme_context_with_media_root_override(self) -> None:
        request = self.factory.get('/')
        cookie_name = get_theme_cookie_name()
        request.COOKIES = {cookie_name: 'material-light'}

        context = theme_context(request)
        data = context['theme']
        self.assertEqual(data['family'], 'material')
        self.assertEqual(data['mode'], 'light')



from django.apps import apps
from django.urls import path, include

app_name = 'django_spire'

urlpatterns = []

for app_config in apps.get_app_configs():
    if hasattr(app_config, 'URLPATTERNS_INCLUDE') and hasattr(app_config, 'URLPATTERNS_NAMESPACE'):
        urlpatterns.append(
            path(
                f'{app_config.URLPATTERNS_NAMESPACE}/',
                include(
                    app_config.URLPATTERNS_INCLUDE,
                    namespace=app_config.URLPATTERNS_NAMESPACE,
                )
            ),
        )

if len(urlpatterns) == 0:
    raise Exception('You need to have at least one app installed to use Django Spire.')
